<?php

namespace App\Exports;

use App\Models\DivisionModel;
use Maatwebsite\Excel\Concerns\FromArray;
use Maatwebsite\Excel\Concerns\WithHeadings;
use Maatwebsite\Excel\Concerns\ShouldAutoSize;
use Maatwebsite\Excel\Concerns\WithEvents;
use Illuminate\Support\Facades\Session;
use Maatwebsite\Excel\Events\AfterSheet;
use Illuminate\Support\Facades\DB;
use App\Models\VehicleTrakingReportModel;
use App\Models\VehicleModel;

class DistanceTravelExport implements FromArray,WithHeadings,ShouldAutoSize,WithEvents
{
    protected $data;
    protected $headings;
    protected $columnWidth = [];
    protected $rowHeight = [];
    protected $mergeCells = [];
    protected $font = [];
    protected $fontSize = [];
    protected $bold = [];
    protected $background = []; 
    protected $vertical = []; 
    protected $sheetName;
    protected $borders = [];

    /**
    * @return \Illuminate\Support\Collection
    */
    public function array(): array
    {
        $aData = array();
        $aData['vehicle_id']      = session::has('vehicle_id') ? session::get('vehicle_id') : 0;
        $aData['vehicle_type_id'] = session::has('vehicle_type_id') ? session::get('vehicle_type_id') : 0;
        $aData['start_date']      = session::has('start_date') ? session::get('start_date') : date('d-m-Y',strtotime('-30 days'));
        $aData['end_date']        = session::has('end_date') ? session::get('end_date') : date('d-m-Y',strtotime('now'));
        
		$aData['records']      = VehicleTrakingReportModel::get_distance_travel_data($aData);
        $total_distance_travelled = 0;

        if( $aData['records']){
            foreach($aData['records'] as $key=>$val){ 
                $distance_km = ($val->distance_travelled/1000);
                $total_distance_travelled += $distance_km;	
                    $aData['excel'][] = array(
                        'vehicle_name' => $val->vehicle_name,
                        'distance_travelled' => number_format($distance_km,2).' Km'
                    ); 
            }
            $aData['excel'][] = array(
                'Total' => "Total ",
                'distance_travelled' => number_format($total_distance_travelled,2).' Km'
            ); 
        }else{
            $aData['excel'][] = array(
            );     
        }

        return $aData['excel'];
    }

    public function headings(): array
    {   
        $aData['vehicle_id']      = session::has('vehicle_id') ? session::get('vehicle_id') : 0;
        $aData['vehicle_type_id'] = session::has('vehicle_type_id') ? session::get('vehicle_type_id') : 0;
        $aData['start_date']      = session::has('start_date') ? session::get('start_date') : date('d-m-Y',strtotime('-30 days'));
        $aData['end_date']        = session::has('end_date') ? session::get('end_date') : date('d-m-Y',strtotime('now'));

        $vehicle_number = "-";
        if($aData['vehicle_id'] != 0){
            $vehicle= VehicleModel::where('id', $aData['vehicle_id'])->get('vehicle_number');
            foreach($vehicle as $val){
                    $vehicle_number = $val->vehicle_number;
                }
        }

        $vehicle_types = "-";
        if($aData['vehicle_type_id'] != 0){
                $vehicle = DB::table('vehicle_types')->where('id',  $aData['vehicle_type_id'])->get('type'); 
                foreach($vehicle as $val){
                        $vehicle_types = $val->type;
                }
        }
               
        return [
                ['Report Name: Distance Travel Report.'],
                ['Vehicle Number : '.$vehicle_number ],
                ['Vehicle Type : '.$vehicle_types ],
                ['Start Date : '. $aData['start_date']],
                ['End Date : '. $aData['end_date']],
                [],
                [ 
                    'Vehicle Number ',
                    'Distance Travelled '                
                ]
        ];
    }

      /**
     * @return string
     */
    public function registerEvents(): array
    {
        return [
            AfterSheet::class => function(AfterSheet $event) {
                $event->sheet->getDelegate()->getStyle('A1:Z1265')->getAlignment()->setHorizontal('left');
                $event->sheet->mergeCells('A1:D1');
                $event->sheet->mergeCells('A2:D2');
                $event->sheet->mergeCells('A3:D3');
                $event->sheet->mergeCells('A4:D4');
                $event->sheet->mergeCells('A5:D5');
                $event->sheet->mergeCells('A6:Z6');
                
                $aData = array();
                $aData['vehicle_id']      = session::has('vehicle_id') ? session::get('vehicle_id') : 0;
                $aData['vehicle_type_id'] = session::has('vehicle_type_id') ? session::get('vehicle_type_id') : 0;
                $aData['start_date']      = session::has('start_date') ? session::get('start_date') : date('d-m-Y',strtotime('-30 days'));
                $aData['end_date']        = session::has('end_date') ? session::get('end_date') : date('d-m-Y',strtotime('now'));
                $aData['records']      = VehicleTrakingReportModel::get_distance_travel_data($aData);
                $i = 8;
               
                if( $aData['records']){
                    $data = count($aData['records']);
                    $i += $data;
                    $event->sheet->getStyle('A'.$i.':Z'.$i)->applyFromArray([
                        'font'=>[
                                'bold'=>true,
                        ]
                    ]);
                    $event->sheet->getDelegate()->getStyle('A'.$i.':A'.$i)->getAlignment()->setHorizontal('right');
                }

                $event->sheet->getDelegate()->getRowDimension('1')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('2')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('3')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('4')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('5')->setRowHeight(25);
                $event->sheet->getDelegate()->getRowDimension('7')->setRowHeight(25);

              
                $event->sheet->getStyle('A1:Z7')->applyFromArray([
                    'font'=>[
                            'bold'=>true,
                    ]
                ]);
            },
        ];
    }
  
}

